﻿using System;
using Microsoft.Scripting.Hosting;
using Ruby;
using Ruby.Runtime;

namespace C_IronRuby
{
    class Program
    {
        static void Main(string[] args)
        {
            DLRHost host = new DLRHost();
            host.DoWork();

            Console.Read();
        }
    }

    internal class DLRHost
    {
        private ScriptEngine engine;
        private ScriptScope scope;

        private MyClass my_class;

        public void DoWork()
        {
            ScriptRuntime runtime = IronRuby.CreateRuntime();
            engine = IronRuby.GetEngine(runtime);
            RubyExecutionContext ctx = IronRuby.GetExecutionContext(engine);

            my_class = new MyClass();
            ctx.DefineGlobalVariable("my_class", my_class);

            string startingBlock = "puts 'Hello From Ruby'" + Environment.NewLine + 
                                   "$my_class.OnChanged('Hello From Ruby')";

            ScriptSource source = engine.CreateScriptSourceFromString(startingBlock);
            source.Execute();
        }

// Doesn't work with IronRuby        
//        public void DoWork2()
//        {
//            ScriptRuntime runtime = IronRuby.CreateRuntime();
//            engine = IronRuby.GetEngine(runtime);

//            string startingBlock = @"class UserGroup
//                                         attr_accessor :name
//                                         
//                                         def to_s()
//                                           name
//                                         end
//                                         
//                                         def GetName()
//                                            name
//                                         end
//                                     end
//                                    $x = UserGroup.new
//                                    $x.name = ""NxtGen""
//                                    print 'Ruby:', $x
//                                    ";

//            scope = engine.CreateScope();
//            //Execute the above code
//            scope.Execute(startingBlock);

//            //Now get access
//            RubyExecutionContext ctx = IronRuby.GetExecutionContext(engine);
//            object rubyX = ctx.GetGlobalVariable("x");

//            IList<string> names = engine.Operations.GetMemberNames(rubyX);

//            Func<string> getName = engine.Operations.GetMember<Func<string>>(rubyX, "GetName");

//            // Now call fooTest's object member function 'f'
//            string result = getName();
//            Console.WriteLine("C#: " + result);
//        }
    }

    public class MyClass
    {
        public delegate void ChangedEventHandler(object sender, MyEventArgs e);
        public event ChangedEventHandler Changed;

        public MyClass()
        {
            Changed += MyClass_Changed;
        }

        void MyClass_Changed(object sender, MyEventArgs e)
        {
            Console.WriteLine("I've been called (This is C#) " + e.DataFromRuby);
        }

        protected virtual void OnChanged(string data)
        {
            if (Changed != null)
            {
                var args = new MyEventArgs();
                args.DataFromRuby = data;
                Changed(this, args);
            }
        }

    }

    public class MyEventArgs : EventArgs
    {
        public string DataFromRuby { get; set; }
    }
}
